from __future__ import annotations
from typing import Dict, Any
import dataclasses
from dacite import from_dict
from .position_book_bucket import PositionBookBucket
from typing import List, Optional, TypeVar

T = TypeVar("T", bound="PositionBook")


@dataclasses.dataclass
class PositionBook:
    """The representation of an instrument's position book at a point in time

    Attributes:
        instrument (Union[Unset, str]): The position book's instrument
        time (Union[Unset, str]): The time when the position book snapshot was created
        price (Union[Unset, str]): The price (midpoint) for the position book's instrument at the time of the position
            book snapshot
        bucket_width (Union[Unset, str]): The price width for each bucket. Each bucket covers the price range from the
            bucket's price to the bucket's price + bucketWidth.
        buckets (Union[Unset, List['PositionBookBucket']]): The partitioned position book, divided into buckets using a
            default bucket width. These buckets are only provided for price ranges which actually contain order or position
            data."""

    instrument: Optional[str]
    time: Optional[str]
    price: Optional[str]
    bucket_width: Optional[str]
    buckets: Optional[List["PositionBookBucket"]]

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "PositionBook":
        """Create a new instance from a dictionary."""
        return from_dict(data_class=cls, data=data)
